RULES_SUBCLASSES = [
    'AIImageElementRules',
    'AIVideoElementRules',
    'ImageElementRules',
    'VideoElementRules',
    'StockElementRules',
    'CustomStockElementRules',
    'MemeElementRules',
    'SoundElementRules',
    'YoutubeVideoElementRules',
    'TextElementRules',
    'PremadeElementRules',
    'EffectElementRules',
    'GreenscreenElementRules'
]

BUILDER_SUBCLASSES = [
    'ImageElementBuilder',
    'AIImageElementBuilder',
    'VideoElementBuilder',
    'AIVideoElementBuilder',
    'CustomStockElementBuilder',
    'StockElementBuilder',
    'SoundElementBuilder',
    'MemeElementBuilder',
    'YoutubeVideoElementBuilder',
    'TextElementBuilder',
    'PremadeElementBuilder'
]